<?php
/* $Id: pgsql.php 70 2010-06-12 17:17:47Z mir $ */

include_once 'config.inc.php';
require_once 'persistens.php';

class Pgsql extends PDO implements WebcalSupport {

	private $user;

    function __construct($dns = NULL) {
		if ($dns)
			$db_dns = "pgsql:$dns";
		else
			$db_dns = 'pgsql:'.DNS;
        try {
            parent::__construct($db_dns);
            $locale = iconv_get_encoding('internal_encoding');
            parent::exec("set client_encoding = '$locale'");
        }
        catch (PDOException $ex) {
            throw new Exception($ex->getMessage());
        }
    }

	private function getMsg($resource) {
		$err = $resource->errorInfo();
		if (count($err) > 2)
			return $err[2];
		else
			return "";
	}  

	function initDatabase($name, $pwd, $uid) {
	    $usr = "create user $uid with encrypted password '$pwd'";
    	$init = "create database $name with encoding 'utf8' template template0 owner $uid";
		try {
			$this->exec($usr);
			$this->exec($init);
		}
		catch (PDOException $ex) {
            throw new Exception($ex->getMessage());
        }
	}

    function createDatabase($name) {
        $sql = "select count(*) as exist from pg_database where datname=?";
        $sth = $this->prepare($sql);
        $sth->execute(array($name));
        $res = $sth->fetch();
        if ($res['exist'] == 0) {
   			$this->beginTransaction();
            include_once 'db_create.postgresql.php';
            $db = make_sql_stm();
            foreach ($db as $sql) {
                $sth->closeCursor();
                $sth = $this->prepare($sql);
                if (! $sth) {
                    $this->rollBack();
                    $err = "$sql\n";
                    foreach ($this->errorInfo() as $info)
                        $err .= "$info\n";
                    throw new Exception($err);
                }
              	$sth->execute();
                //throw new Exception(var_export($sth, true));
                if (! $sth) {
                    $this->rollBack();
                    $err = "$sql\n";
                    foreach ($this->errorInfo() as $info)
                        $err .= "$info\n";
                    throw new Exception($err);
                }
			}
            $this->commit();
        }
        else
            throw new Exception("Database exists");
    }

    function getViewStyle($uid) {
        $sql = "select userview from users u where u.uid = ?";
        $sth = $this->prepare($sql);
        $sth->execute(array($uid));
        $res = $sth->fetch();
        return $res[0];
    }

    function getRole($uid) {
        $sql = "select userrole from users u where u.uid = ?";
        $sth = $this->prepare($sql);
        $sth->execute(array($uid));
        $res = $sth->fetch();
        return $res[0];
    }

    function getTimeout($uid) {
        $sql = "select timeout from users u where u.uid = ?";
        $sth = $this->prepare($sql);
        $sth->execute(array($uid));
        $res = $sth->fetch();
        return $res[0];
    }

    function getTimezone($uid) {
        $sql = "select timezone from users u where u.uid = ?";
        $sth = $this->prepare($sql);
        $sth->execute(array($uid));
        $res = $sth->fetch();
        return $res[0];
    }

    function getStartWeek($uid) {
        $sql = "select weekstart from users u where u.uid = ?";
        $sth = $this->prepare($sql);
        $sth->execute(array($uid));
        $res = $sth->fetch();
        return ($res[0]) ? 'MO' : 'SU';
    }

    function getStartHour($uid) {
        $sql = "select daystart from users u where u.uid = ?";
        $sth = $this->prepare($sql);
        $sth->execute(array($uid));
        $res = $sth->fetch();
        return $res[0];
    }

    function getEndHour($uid) {
        $sql = "select daystart from users u where u.uid = ?";
        $sth = $this->prepare($sql);
        $sth->execute(array($uid));
        $res = $sth->fetch();
        return $res[0];
    }

    function authenticate($uid, $pwd) {
        $res = array();
        $sql = "select userrole, timeout, userview, weekstart, daystart,
        dayend, timezone, seckey, pubkey, c.id as id, name, color, config
        from users u left join calendar c on c.uid = u.id where
        u.uid = ? and u.pwd = ?";
        $sth = $this->prepare($sql);
        if (! $sth) {
            $err = "$sql\n";
            foreach ($this->errorInfo() as $info)
                $err .= "$info\n";
            throw new Exception($err);
        }
        $sth->execute(array($uid,$pwd));
        if (! $sth) {
            $err = "$sql\n";
            foreach ($this->errorInfo() as $info)
                $err .= "$info\n";
            throw new Exception($err);
        }
        $res = $sth->fetchAll(PDO::FETCH_ASSOC);
        
        /*
        if ($res[0]['config']) {
            //$res[0]['config'] = base64_decode($res[0]['config']);
            //$res[0]['config'] = quoted_printable_decode($res[0]['config']);
            $res[0]['config'] = $res[0]['config'];
		}
		*/

        return $res;
    }

    function addUser($data) {
        //$fp = fopen('/tmp/davical.log', 'a');
        //fwrite($fp, "New user\nuid: {$data['uid']}\npwd: {$data['pwd']}\ntimezone: {$data['timezone']}\nuserrole: {$data['userrole']}\nseckey: {$data['seckey']}\npubkey: {$data['pubkey']}");
        //fclose($fp);
        $sql = "insert into users (uid, pwd, timezone, userrole, seckey, pubkey)
            values (?,?,?,?,?,?)";
        $sth = $this->prepare($sql);
        if (! $sth) {
            foreach ($this->errorInfo() as $info)
                $err .= "$info\n";
            return $err;
        }
        $this->beginTransaction();
        $sth->execute(array($data['uid'],$data['pwd'],$data['timezone'],
            $data['userrole'],$data['seckey'],$data['pubkey']));
        if (! $sth) {
            $this->rollBack();
            $err = "$sql\n";
            foreach ($this->errorInfo() as $info)
                $err .= "$info\n";
            return $err;
        }
        $this->commit();
        return TRUE;
    }

    function setUserSettings($uid, $data) {
        $sql = "update users set userrole = ?, timeout = ?, userview = ?,
            weekstart = ?, daystart = ?, dayend = ?, timezone = ?
            where uid = ?";
        $sth = $this->prepare($sql);
        if (! $sth) {
            $err = "$sql\n";
            foreach ($this->errorInfo() as $info)
                $err .= "$info\n";
            return $err;
        }
        $this->beginTransaction();
        $sth->execute(array($data['userrole'],$data['timeout'],
            $data['userview'],$data['weekstart'],$data['daystart'],
            $data['dayend'],$data['timezone'],$uid));
        if (! $sth) {
            $this->rollBack();
            $err = "$sql\n";
            foreach ($this->errorInfo() as $info)
                $err .= "$info\n";
            return $err;
        }
        $this->commit();
        return TRUE;
    }
/*
timeout integer default 3600,
userview text default 'week',
weekstart integer default 0,
daystart real default 8.00,
dayend real default 17.00,
timezone text default 'Etc/UTC',
*/
    function changeDefault($data) {
        $temp = <<<_TEMP
            CREATE TEMPORARY TABLE tmpuser (
                id integer,
                uid text,
                pwd text,
                userrole integer,
                timeout integer,
                userview text,
                weekstart integer,
                daystart real,
                dayend real,
                timezone text,
                seckey text,
                pubkey text
            )
_TEMP;
        $table = <<<_TABLE
            CREATE TABLE users (
                id integer primary key autoincrement,
                uid text not null,
                pwd text not null,
                userrole integer not null,
                timeout integer default __TIMEOUT__,
                userview text default '__VIEW__',
                weekstart integer default __WEEKSTART__,
                daystart real default __DAYSTART__,
                dayend real default __DAYEND__,
                timezone text default '__TIMEZONE__',
                seckey text default '',
                pubkey text default '',
                constraint uid_index unique (uid)
                constraint userrole_fk foreign key (userrole) references roles (id)
                on delete restrict
                on update cascade
                on insert cascade
            )
_TABLE;
        $table = str_replace('__TIMEOUT__', $data['TIMEOUT'], $table);
        $table = str_replace('__VIEW__', $data['VIEW_STYLE'], $table);
        $table = str_replace('__WEEKSTART__', $data['WEEK_START_SUNDAY'], $table);
        $table = str_replace('__DAYSTART__', $data['START_HOUR'], $table);
        $table = str_replace('__DAYEND__', $data['END_HOUR'], $table);
        $table = str_replace('__TIMEZONE__', $data['TIMEZONE'], $table);
        $this->beginTransaction();
        if ($this->exec($temp) === FALSE) {
            $err = "$sql\n";
            foreach ($this->errorInfo() as $info)
                $err .= "$info\n";
            $this->rollBack();
            return $err;
        }
        if ($this->exec('insert into tmpuser select * from users') === FALSE) {
            $err = "cp to tmp table\n";
            foreach ($this->errorInfo() as $info)
                $err .= "$info\n";
            $this->rollBack();
            return $err;
        }
        if ($this->exec('drop table users') === FALSE) {
            $err = "drop users table\n";
            foreach ($this->errorInfo() as $info)
                $err .= "$info\n";
            $this->rollBack();
            return $err;
        }
        if ($this->exec($table) === FALSE) {
            $err = "create table\n";
            foreach ($this->errorInfo() as $info)
                $err .= "$info\n";
            $this->rollBack();
            return $err;
        }
        if ($this->exec('insert into users select * from tmpuser') === FALSE) {
            $err = "cp tmp to users table\n";
            foreach ($this->errorInfo() as $info)
                $err .= "$info\n";
            $this->rollBack();
            return $err;
        }
        if ($this->exec('drop table tmpuser') === FALSE) {
            $err = "drop users table\n";
            foreach ($this->errorInfo() as $info)
                $err .= "$info\n";
            $this->rollBack();
            return $err;
        }
        $this->commit();
        return TRUE;
    }

    function deleteUser($uid) {
        $sql = "delete from users where uid = ?";
        $sth = $this->prepare($sql);
        if (! $sth) {
            $err = "$sql\n";
            foreach ($this->errorInfo() as $info)
                $err .= "$info\n";
            return $err;
        }
        $this->beginTransaction();
        $sth->execute(array($uid));
        if (! $sth) {
            $this->rollBack();
            $err = "$sql\n";
            foreach ($this->errorInfo() as $info)
                $err .= "$info\n";
            return $err;
        }
        $this->commit();
        return TRUE;
    }

    function addCalendar($uid, CalendarInfo $cal) {
        //$fp = fopen('/tmp/add_calender.log', 'a');
        //fwrite($fp, "New calendar\nUID: $uid\nname: {$cal->name}\nColor: {$cal->color}\nConfig: {$cal->config}\n");
        $this->beginTransaction();
        $sql = "select id from users where uid = ?";
        //fwrite($fp, "Get ID: $sql\n");
        $sth = $this->prepare($sql);
        if (! $sth) {
            $err = "$sql\n";
            foreach ($this->errorInfo() as $info)
                $err .= "$info\n";
            $this->rollBack();
            return $err;
        }
        $sth->execute(array($uid));
        if (! $sth) {
            $err = "$sql\n";
            foreach ($this->errorInfo() as $info)
                $err .= "$info\n";
            $this->rollBack();
            return $err;
        }
        $sth->bindColumn(1, $id);
        if (! $sth) {
            $err = "$sql\n";
            foreach ($this->errorInfo() as $info)
                $err .= "$info\n";
            $this->rollBack();
            return $err;
        }
        $sth->fetch(PDO::FETCH_BOUND);
        //fwrite($fp, "Returned users ID: $id\n");
        if (! $sth) {
            $err = "$sql\n";
            foreach ($this->errorInfo() as $info)
                $err .= "$info\n";
            $this->rollBack();
            return $err;
        }
        $sth->closeCursor();
        $sql = "insert into calendar (uid, name, color, config)
            values ($id,?,?,?)";
        //var_dump($sql);
        //fwrite($fp, "Insert into calendar: $sql\n");
        $sth = $this->prepare($sql);
        if (! $sth) {
            $err = "$sql\n";
            foreach ($this->errorInfo() as $info)
                $err .= "$info\n";
            $this->rollBack();
            return $err;
        }
        //$sth->execute(array($cal->name,$cal->color,base64_encode($cal->config)));
        $sth->execute(array($cal->name, $cal->color, $cal->config));
        if (! $sth) {
            $err = "$sql\n";
            foreach ($this->errorInfo() as $info)
                $err .= "$info\n";
            $this->rollBack();
            return $err;
        }
        $sth->closeCursor();
        $sth = $this->prepare('select max(id) from calendar');
        $sth->execute();
        if (! $sth) {
            $err = "$sql\n";
            foreach ($this->errorInfo() as $info)
                $err .= "$info\n";
            $this->rollBack();
            return $err;
        }
        $sth->bindColumn(1, $id);
        if (! $sth) {
            $err = "$sql\n";
            foreach ($this->errorInfo() as $info)
                $err .= "$info\n";
            $this->rollBack();
            return $err;
        }
        $sth->fetch(PDO::FETCH_BOUND);
        //fwrite($fp, "Returned ID for created calendar: $id\n");
        if (! $sth) {
            $err = "$sql\n";
            foreach ($this->errorInfo() as $info)
                $err .= "$info\n";
            $this->rollBack();
            return $err;
        }
        $sth->closeCursor();
        $this->commit();
/*        $sql = "select * from calendar where id = $id";
        $sth = $this->prepare($sql);
        $sth->execute();
        $result = $sth->fetchAll(PDO::FETCH_ASSOC);
        if ($result)
            fwrite($fp, var_export($result[0], TRUE) . "\n");
        else
            fwrite($fp, "No date found in calendar relation\n");
        fclose($fp);*/
        return $id;
    }

    function deleteCalendar($uid, $id) {
        $this->beginTransaction();
        $sql = "select id from users where uid = ?";
        $sth = $this->prepare($sql);
        if (! $sth) {
            $err = "$sql\n";
            foreach ($this->errorInfo() as $info)
                $err .= "$info\n";
            $this->rollBack();
            return $err;
        }
        $sth->execute(array($uid));
        if (! $sth) {
            $err = "$sql\n";
            foreach ($this->errorInfo() as $info)
                $err .= "$info\n";
            $this->rollBack();
            return $err;
        }
        $sth->bindColumn(1, $uid);
        if (! $sth) {
            $err = "$sql\n";
            foreach ($this->errorInfo() as $info)
                $err .= "$info\n";
            $this->rollBack();
            return $err;
        }
        $sth->fetch(PDO::FETCH_BOUND);
        if (! $sth) {
            $err = "$sql\n";
            foreach ($this->errorInfo() as $info)
                $err .= "$info\n";
            $this->rollBack();
            return $err;
        }
        $sth->closeCursor();
        $sql = "delete from calendar where id = ? and uid = $uid";
        $sth = $this->prepare($sql);
        if (! $sth) {
            $err = "$sql\n";
            foreach ($this->errorInfo() as $info)
                $err .= "$info\n";
            $this->rollBack();
            return $err;
        }
        $sth->execute(array($id));
        if (! $sth) {
            $err = "$sql\n";
            foreach ($this->errorInfo() as $info)
                $err .= "$info\n";
            $this->rollBack();
            return $err;
        }
        $this->commit();
        return TRUE;
    }

    function updateCalendar($uid, $id, CalendarInfo $cal) {
        $this->beginTransaction();
        $sql = "select id from users where uid = ?";
        $sth = $this->prepare($sql);
        if (! $sth) {
            $err = "$sql\n";
            foreach ($this->errorInfo() as $info)
                $err .= "$info\n";
            $this->rollBack();
            return $err;
        }
        $sth->execute(array($uid));
        if (! $sth) {
            $err = "$sql\n";
            foreach ($this->errorInfo() as $info)
                $err .= "$info\n";
            $this->rollBack();
            return $err;
        }
        $sth->bindColumn(1, $uid);
        if (! $sth) {
            $err = "$sql\n";
            foreach ($this->errorInfo() as $info)
                $err .= "$info\n";
            $this->rollBack();
            return $err;
        }
        $sth->fetch(PDO::FETCH_BOUND);
        if (! $sth) {
            $err = "$sql\n";
            foreach ($this->errorInfo() as $info)
                $err .= "$info\n";
            $this->rollBack();
            return $err;
        }
        $sth->closeCursor();
        $sql = "update calendar set name = ?, color = ?, config = ?
        where id = ? and uid = $uid";
        $sth = $this->prepare($sql);
        if (! $sth) {
            $err = "$sql\n";
            foreach ($this->errorInfo() as $info)
                $err .= "$info\n";
            $this->rollBack();
            return $err;
        }
        //$sth->execute(array($cal->name,$cal->color,base64_encode($cal->config),$id));
        $sth->execute(array($cal->name,$cal->color, $cal->config,$id));
        if (! $sth) {
            $err = "$sql\n";
            foreach ($this->errorInfo() as $info)
                $err .= "$info\n";
            $this->rollBack();
            return $err;
        }
        $this->commit();
        return TRUE;
    }

    function changePassword($uid, $pwd) {
        $this->beginTransaction();
        $sql = "update users set pwd = ? where uid = ?";
        $sth = $this->prepare($sql);
        if (! $sth) {
            $err = "$sql\n";
            foreach ($this->errorInfo() as $info)
                $err .= "$info\n";
            return $err;
        }
        $sth->execute(array($pwd,$uid));
        if (! $sth) {
            $this->rollBack();
            $err = "$sql\n";
            foreach ($this->errorInfo() as $info)
                $err .= "$info\n";
            return $err;
        }
        $this->commit();
        return TRUE;
    }

    function getAllUsers($limit, $offset) {
		if ($limit == -1) {
        	$sql = "select u.uid, r.name as userrole from users u, roles r
        	where u.userrole = r.id limit all offset ?";
		}
		else {
        	$sql = "select u.uid, r.name as userrole from users u, roles r
        	where u.userrole = r.id limit ? offset ?";
		}
        $sth = $this->prepare($sql);
        if (! $sth) {
            $err = "$sql\n";
            foreach ($this->errorInfo() as $info)
                $err .= "$info\n";
            return $err;
        }
        if ($limit == -1)
        	$sth->execute(array($offset));
        else
        	$sth->execute(array($limit,$offset));
        if (! $sth) {
            $err = "$sql\n";
            foreach ($this->errorInfo() as $info)
                $err .= "$info\n";
            return $err;
        }
        return $sth->fetchAll(PDO::FETCH_ASSOC);
    }

    function getRoles() {
        $sql = "select id, name from roles order by id desc";
        $sth = $this->prepare($sql);
        if (! $sth) {
            $err = "$sql\n";
            foreach ($this->errorInfo() as $info)
                $err .= "$info\n";
            return $err;
        }
        $sth->execute();
        if (! $sth) {
            $err = "$sql\n";
            foreach ($this->errorInfo() as $info)
                $err .= "$info\n";
            return $err;
        }
        return $sth->fetchAll(PDO::FETCH_ASSOC);
    }

    function getRoleName($id) {
        $sql = "select name from roles where id = ?";
        $sth = $this->prepare($sql);
        if (! $sth) {
            $err = "$sql\n";
            foreach ($this->errorInfo() as $info)
                $err .= "$info\n";
            return $err;
        }
        $sth->execute(array($id));
        if (! $sth) {
            $err = "$sql\n";
            foreach ($this->errorInfo() as $info)
                $err .= "$info\n";
            return $err;
        }
        return $sth->fetch(PDO::FETCH_ASSOC);
    }

    function getVersion() {
        $version = array('version' => 0);
        $sql = "select count(*) as exist from pg_tables where tablename = 'about'";
        $sth = $this->prepare($sql);
        if (! $sth) {
            $err = "$sql\n";
            foreach ($this->errorInfo() as $info)
                $err .= "$info\n";
            return $err;
        }
        $sth->execute();
        if (! $sth) {
            $err = "$sql\n";
            foreach ($this->errorInfo() as $info)
                $err .= "$info\n";
            return $err;
        }
        $res = $sth->fetch();
        if ($res['exist'] > 0) {
            $sql = "select version from about where id = 1";
            $sth = $this->prepare($sql);
            if (! $sth) {
                $err = "$sql\n";
                foreach ($this->errorInfo() as $info)
                    $err .= "$info\n";
                return $err;
            }
            $sth->execute();
            if (! $sth) {
                $err = "$sql\n";
                foreach ($this->errorInfo() as $info)
                    $err .= "$info\n";
                return $err;
            }
            $res = $sth->fetch();
            $version = $res;
        }
        return $version;
    }

    function execute($sql) {
        $sth = $this->prepare($sql);
        if (! $sth) {
            $err = "$sql\n";
            foreach ($this->errorInfo() as $info)
                $err .= "$info\n";
            return $err;
        }
        $sth->execute();
        if (! $sth) {
            $err = "$sql\n";
            foreach ($this->errorInfo() as $info)
                $err .= "$info\n";
            return $err;
        }
        return true;
    }

    function getCalendarConfig($id) {
		if ($id == -1) {
			$sql = "select id, config from calendar";
		}
		else {
			$sql = "select id, config from calendar where id = $id";
		}
        $sth = $this->prepare($sql);
        if ($sth->errorCode() && $sth->errorCode() != '00000') {
            $err = "$sql\n";
       		$err .= $this->getMsg($sth)."\n";
            return $err;
        }
        $sth->execute();
        if ($sth->errorCode() && $sth->errorCode() != '00000') {
            $err = "$sql\n";
       		$err .= $this->getMsg($sth)."\n";
            return $err;
        }
        return $sth->fetchAll(PDO::FETCH_ASSOC);
	}
	
}

?>
